﻿--[[
Grid2StatusParty.lua
Add grid statuses for party leader, raid leader, raid assistant, and master looter

This file is covered under the BSD-2.0 license.  See LICENSE.TXT for details
Authors: Byron Shelden (Dashkal), Pastamancer, Grid2 conversion by Toadkiller
--]]

local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Grid2")
local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

local Grid2StatusGroup = Grid2:NewModule("StatusGroup")

local GroupLeader = Grid2.statusPrototype:new("leader")
local GroupAssistant = Grid2.statusPrototype:new("raid-assistant")
local MasterLooter = Grid2.statusPrototype:new("master-looter")

Grid2 = Grid2



-- function GroupLeader:UpdateUnit(event, unit)
	-- self:UpdateIndicators(unit)
-- end

function GroupLeader:UpdateAllUnits(event)
	for unit, guid in Grid2:IterateRosterUnits() do
		self:UpdateIndicators(unit)
	end
end

function GroupLeader:OnEnable()
	self:RegisterEvent("PARTY_LEADER_CHANGED", "UpdateAllUnits")
	-- self:RegisterMessage("Grid_UnitJoined", "UpdateUnit")
end

function GroupLeader:OnDisable()
	self:UnregisterEvent("PARTY_LEADER_CHANGED")
	-- self:UnregisterMessage("Grid_UnitJoined")
end

function GroupLeader:IsActive(unit)
	if (UnitIsPartyLeader(unit)) then
		return true
	end
	return nil
end

function GroupLeader:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function GroupLeader:GetIcon(unit)
	return "Interface\\GroupFrame\\UI-Group-LeaderIcon"
end

function GroupLeader:GetText(unit)
	if (GetNumRaidMembers() > 0) then
		return L["RL"]
	else
		return L["PL"]
	end
end

local function CreateGroupLeader(baseKey, dbx)
	Grid2:RegisterStatus(GroupLeader, {"color", "icon", "text"}, baseKey, dbx)

	return GroupLeader
end

Grid2.setupFunc["leader"] = CreateGroupLeader



-- function GroupAssistant:UpdateUnit(event, unit)
	-- self:UpdateIndicators(unit)
-- end

function GroupAssistant:UpdateAllUnits(event)
	for unit, guid in Grid2:IterateRosterUnits() do
		self:UpdateIndicators(unit)
	end
end

function GroupAssistant:OnEnable()
	self:RegisterEvent("PARTY_LEADER_CHANGED", "UpdateAllUnits")
	-- self:RegisterMessage("Grid_UnitJoined", "UpdateUnit")
end

function GroupAssistant:OnDisable()
	self:UnregisterEvent("PARTY_LEADER_CHANGED")
	-- self:UnregisterMessage("Grid_UnitJoined")
end

-- lookup for Raid
local unitsRaidindex = {}
for i = 1, MAX_RAID_MEMBERS do
	unitsRaidindex["raid" .. i] = i
end

function GroupAssistant:IsActive(unit)
	local raidIndex = unitsRaidindex[unit]
	if (raidIndex) then
		local name, rank = GetRaidRosterInfo(raidIndex)
		return rank == 1
	end
end

function GroupAssistant:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function GroupAssistant:GetIcon(unit)
	return "Interface\\GroupFrame\\UI-Group-AssistantIcon"
end

function GroupAssistant:GetText(unit)
	return L["RA"]
end

local function CreateGroupAssistant(baseKey, dbx)
	Grid2:RegisterStatus(GroupAssistant, {"color", "icon", "text"}, baseKey, dbx)

	return GroupAssistant
end

Grid2.setupFunc["raid-assistant"] = CreateGroupAssistant



-- function MasterLooter:UpdateUnit(event, unit)
	-- self:UpdateIndicators(unit)
-- end

function MasterLooter:UpdateAllUnits(event)
	for unit, guid in Grid2:IterateRosterUnits() do
		self:UpdateIndicators(unit)
	end
end

function MasterLooter:OnEnable()
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED", "UpdateAllUnits")
	-- self:RegisterMessage("Grid_UnitJoined", "UpdateUnit")
end

function MasterLooter:OnDisable()
	self:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED")
	-- self:UnregisterMessage("Grid_UnitJoined")
end

-- lookup for Party
local unitsParty = { [0] = "player" }
for i = 1, MAX_PARTY_MEMBERS do
	unitsParty[i] = "party" .. i
end

-- lookup for Raid
local unitsRaid = {}
for i = 1, MAX_RAID_MEMBERS do
	unitsRaid[i] = "raid" .. i
end

function MasterLooter:IsActive(unit)
	local lootMethod, partyIndex, raidIndex = GetLootMethod()

	if (lootMethod == "master") then
		if (partyIndex) then
			return unit == unitsParty[partyIndex]
		elseif (raidIndex) then
			return unit == unitsRaid[raidIndex] 
		end
	end
	return nil
end

function MasterLooter:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function MasterLooter:GetIcon(unit)
	return "Interface\\GroupFrame\\UI-Group-MasterLooter"
end

function MasterLooter:GetText(unit)
	return L["ML"]
end

local function CreateMasterLooter(baseKey, dbx)
	Grid2:RegisterStatus(MasterLooter, {"color", "icon", "text"}, baseKey, dbx)

	return MasterLooter
end

Grid2.setupFunc["master-looter"] = CreateMasterLooter



-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2StatusGroupOptions", dblData, nil, "account", 1) or upgrade

	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)

	local Grid2StatusGroupOptions = Grid2Options.plugins["Grid2StatusGroupOptions"]
	if (Grid2StatusGroupOptions) then
		flatten = DBL:UpgradeDefaults("Grid2StatusGroupOptions", dblData, Grid2StatusGroupOptions.UpgradeDefaults, "account", 1) or flatten
	end
	
	return flatten
end


